module mabio
use mkinds

!--------------------------
! version 0.1 -- 2013/03/02
! ashwanth srinivasan
! Tendral LLC, Key Biscayne
! -------------------------
! A module to handle hycom .ab style files. The .a is written as a series of
! records - the actual array data is padded by nan values to make the record 
! length a integer multiple of 4096 bytes. The .b files contain summary info 
! for each record in the .a file.
! subroutines:
! zaiost -- initialize the .ab io environemnt
! zaiopn -- open a file for abio
! zaiord -- read a record and generate summary info to compare with the bfile
! zaiordd --file level read of a single record from the .a unit
! zaiowr -- write a single record and generate summary info for the .b file
! zaiowrdd -- write a single record to the iounit
! zaiosk -- skip a single record from the .a file
! rdbr -- read a single line from the bfile
! wrbr -- write a single line to a bfile

implicit none
integer(i4),private                  :: idm,ibdm
integer(i4),private                  :: jdm,jbdm
integer(i4),private, save            :: n2drec,bn2drec
integer(i4),private, save            :: nrecl,bnrecl
integer(i4),private, save            :: iarec(999),ibrec(999)
real(r4),   private, allocatable     :: w(:),wb(:)
real(r4),parameter,private           :: spval=2.00**100

contains

!### initialize hycom format io
subroutine zaiost(nx,ny)
implicit none
integer(i4), intent(in) :: nx
integer(i4), intent(in) :: ny
idm=nx
jdm=ny
!n2drec = size of output 2-d array, multiple of 4096
n2drec = ((idm*jdm+4095)/4096)*4096
!initialize I/O buffer
iarec=-1
if(.not. allocated(w)) then
    allocate( w(n2drec) )
endif
inquire(iolength=nrecl) w
end subroutine zaiost

!### initialize hycom format io
subroutine zbiost(nx,ny)
implicit none
integer(i4), intent(in) :: nx
integer(i4), intent(in) :: ny
ibdm=nx
jbdm=ny
!n2drec = size of output 2-d array, multiple of 4096
bn2drec = ((ibdm*jbdm+4095)/4096)*4096
!initialize I/O buffer
ibrec=-1
if(.not. allocated(wb)) then
    allocate( wb(bn2drec) )
endif
inquire(iolength=bnrecl) wb
!print *, bnrecl
end subroutine zbiost

!### open a .[ab] file 
subroutine zaiopn(flnm,iunit)
character(len=*), intent(in)              :: flnm
integer(i4),      intent(in)              :: iunit
integer(i4)                               :: ios
integer(i4)                               :: iaunit
character(:),allocatable                  :: aflnm
character(:),allocatable                  :: bflnm
iaunit=1000+iunit
iarec(iunit)=0
! open the .a file
aflnm=trim(adjustl(flnm))//".a"
open(unit=iaunit,file=aflnm,form='unformatted',status='unknown',access='direct', recl=nrecl, iostat=ios)
! open the .b file
bflnm=trim(adjustl(flnm))//".b"
open(unit=iunit,file=bflnm,form='formatted',status='unknown',access='sequential',iostat=ios)
end subroutine

subroutine zbiopn(flnm,iunit)
character(len=*), intent(in)              :: flnm
integer(i4),      intent(in)              :: iunit
integer(i4)                               :: ios
integer(i4)                               :: iaunit
character(:),allocatable                  :: aflnm
character(:),allocatable                  :: bflnm
iaunit=1000+iunit
ibrec(iunit)=0
! open the .a file
aflnm=trim(adjustl(flnm))//".a"
open(unit=iaunit,file=aflnm,form='unformatted',status='unknown',access='direct', recl=bnrecl, iostat=ios)
! open the .b file
bflnm=trim(adjustl(flnm))//".b"
open(unit=iunit,file=bflnm,form='formatted',status='unknown',access='sequential',iostat=ios)
end subroutine

!### read an array record from a .ab file
subroutine zaiord(iunit,h,hmin,hmax)
implicit none
integer(i4), intent(in)                            :: iunit
real(r4),    intent(out)                           :: hmin
real(r4),    intent(out)                           :: hmax
real(r4),    dimension (1:idm,1:jdm),intent(out)   :: h
integer(i4)                                        :: ios
integer(i4)                                        :: i
integer(i4)                                        :: j
integer(i4)                                        :: irec
integer(i4)                                        :: iaunit
real(r4)                                           :: wmin
real(r4)                                           :: wmax

iaunit=iunit+1000
iarec(iunit)=iarec(iunit)+1
call zaiordd(w,n2drec,iaunit,iarec(iunit),ios)
wmin =  spval
wmax = -spval
do j= 1,jdm
   do i= 1,idm
   h(i,j) = w(i+(j-1)*idm)
   if (w(i+(j-1)*idm).ne.spval) then
      wmin = min( wmin, w(i+(j-1)*idm) )
      wmax = max( wmax, w(i+(j-1)*idm) )
   endif
  enddo
enddo
hmin = wmin
hmax = wmax
end subroutine

!### low-level read of a single .a record
subroutine zaiordd(a,n,iunit,irec,ios)
implicit none
integer(i4), intent(in)                            :: n
integer(i4), intent(in)                            :: iunit
integer(i4), intent(in)                            :: irec
integer(i4), intent(out)                           :: ios
real(r4),    intent(out)                           :: a(n)
read(unit=iunit, rec=irec, iostat=ios) a
end subroutine zaiordd

!### write a single record and get .b file info
subroutine zaiowr(iunit,h,hmin,hmax)
implicit none
integer(i4), intent(in)                            :: iunit
real(r4),    intent(out)                           :: hmin
real(r4),    intent(out)                           :: hmax
real(r4),dimension (1:idm,1:jdm), intent(inout)    :: h
integer(i4)                                        :: ios
integer(i4)                                        :: i
integer(i4)                                        :: j
integer(i4)                                        :: irec
integer(i4)                                        :: iaunit
real(r4)                                           :: wmin
real(r4)                                           :: wmax
wmin =  spval
wmax = -spval
do j= 1,jdm
  do i= 1,idm
     w(i+(j-1)*idm) = h(i,j)
     if(w(i+(j-1)*idm).ne.spval) then
         wmin = min( wmin, w(i+(j-1)*idm) )
         wmax = max( wmax, w(i+(j-1)*idm) )
     endif
  enddo
enddo
do i= idm*jdm+1,n2drec
    w(i) = spval
enddo
hmin = wmin
hmax = wmax
iaunit=iunit+1000
iarec(iunit) = iarec(iunit) + 1
call zaiowrd(w,n2drec, iaunit,iarec(iunit),ios)
end subroutine zaiowr

subroutine zbiowr(iunit,h,hmin,hmax)
implicit none
integer(i4), intent(in)                            :: iunit
real(r4),    intent(out)                           :: hmin
real(r4),    intent(out)                           :: hmax
real(r4),dimension (1:ibdm,1:jbdm), intent(inout)    :: h
integer(i4)                                        :: ios
integer(i4)                                        :: i
integer(i4)                                        :: j
integer(i4)                                        :: irec
integer(i4)                                        :: iaunit
real(r4)                                           :: wmin
real(r4)                                           :: wmax
wmin =  spval
wmax = -spval
do j= 1,jbdm
  do i= 1,ibdm
     wb(i+(j-1)*ibdm) = h(i,j)
     if(wb(i+(j-1)*ibdm).ne.spval) then
         wmin = min( wmin, wb(i+(j-1)*ibdm) )
         wmax = max( wmax, wb(i+(j-1)*ibdm) )
     endif
  enddo
enddo
do i= ibdm*jbdm+1,bn2drec
    wb(i) = spval
enddo
hmin = wmin
hmax = wmax
iaunit=iunit+1000
ibrec(iunit) = ibrec(iunit) + 1
call zbiowrd(wb,bn2drec, iaunit,ibrec(iunit),ios)
end subroutine zbiowr

!### file level write a single direct access record
subroutine zaiowrd(a,n, iunit,irec,ios)
integer(i4), intent(in)                            :: n
integer(i4), intent(in)                            :: iunit
integer(i4), intent(in)                            :: irec
integer(i4), intent(out)                           :: ios
real(r4),    intent(in)                            :: a(n)
write(unit=iunit, rec=irec, iostat=ios) a
end subroutine zaiowrd

subroutine zbiowrd(a,n, iunit,irec,ios)
integer(i4), intent(in)                            :: n
integer(i4), intent(in)                            :: iunit
integer(i4), intent(in)                            :: irec
integer(i4), intent(out)                           :: ios
real(r4),    intent(in)                            :: a(n)
write(unit=iunit, rec=irec, iostat=ios) a
end subroutine zbiowrd


!### skip a record 
subroutine zaiosk(iunit)
implicit none
integer(i4), intent(in)    :: iunit
iarec(iunit) = iarec(iunit) + 1
end subroutine

!### close the .ab units
subroutine zaiocl(iunit)
implicit none
integer(i4), intent(in)    :: iunit
integer(i4)                :: iaunit
iaunit=iunit+1000
close(iunit)
close(iaunit)
deallocate( w )
end subroutine zaiocl

subroutine zbiocl(iunit)
implicit none
integer(i4), intent(in)    :: iunit
integer(i4)                :: iaunit
iaunit=iunit+1000
close(iunit)
close(iaunit)
deallocate( wb )
end subroutine zbiocl

!### check if a and b files are consistent
subroutine abchk(iunit,l,k,hmna,hmxa,hmnb,hmxb,lfatal)
integer(i4), intent(in) :: iunit
integer(i4), intent(in) :: l
integer(i4), intent(in) :: k
real(r4),    intent(in) :: hmna
real(r4),    intent(in) :: hmxa
real(r4),    intent(in) :: hmnb
real(r4),    intent(in) :: hmxb
logical(bl), intent(in) :: lfatal

! check if a and b files are consistent and stop program if lfatal.
if     (abs(hmna-hmnb).gt.abs(hmnb)*1.e-4 .or. abs(hmxa-hmxb).gt.abs(hmxb)*1.e-4) then
write(*,'(/ a / a,3i3/ a,1p3e14.6 / a,1p3e14.6 /)') &
'error - .a and .b files not consistent:',               &
'iunit,k,l = ',iunit,k,l,                                &
'.a,.b min = ',hmna,hmnb,hmna-hmnb,                      &
'.a,.b max = ',hmxa,hmxb,hmxa-hmxb
if(lfatal) stop '(abchk)'
endif
end subroutine

!### read a .b file record
subroutine rdbr(iunit,iv1,iv2,rv3,rv4,vname)
implicit none
integer(i4),intent(in)              :: iunit
integer(i4),intent(out)             :: iv1
integer(i4),intent(out)             :: iv2
real(r4),intent(out)                :: rv3
real(r4),intent(out)                :: rv4
character(len=8),intent(out)        :: vname
character(len=80)                   :: cline
integer(i4)                         :: j
read(iunit,'(a)') cline
vname=cline(1:8)
j=index(cline,'=')
read(cline(j+1:),*) iv1,rv3,iv2,rv4
end subroutine


! restartb  file record
subroutine rdrstbr(iunit,iv1,iv2,rv3,rv4,vname)
implicit none
integer(i4),intent(in)              :: iunit
integer(i4),intent(out)             :: iv1
integer(i4),intent(out)             :: iv2
real(r4),intent(out)                :: rv3
real(r4),intent(out)                :: rv4
character(len=8),intent(out)        :: vname
character(len=80)                   :: cline
integer(i4)                         :: j
read(iunit,'(a)') cline
!write(*,*) cline
vname=cline(1:8)
j=index(cline,'=')
read(cline(j+1:),*) iv1,iv2,rv3,rv4
end subroutine

!### write a record to the .b file
subroutine wrbr(iunit,iv1,iv2,rv3,rv4,vname)
implicit none
integer(i4),intent(in)             :: iunit
integer(i4),intent(in)             :: iv1
integer(i4),intent(in)             :: iv2
real(r4),intent(in)                :: rv3
real(r4),intent(in)                :: rv4
character(len=8),intent(in)        :: vname
write(iunit,4100) vname,iv1,rv3,iv2,rv4 
4100 format(a,': layer,tlevel,range = ',i9,2x,1p2e16.7,2x,i3,1p2e16.7)
end subroutine

!### write a record to the .b file
subroutine wrrstbr(iunit,iv1,iv2,rv3,rv4,vname)
implicit none
integer(i4),intent(in)             :: iunit
integer(i4),intent(in)             :: iv1
integer(i4),intent(in)             :: iv2
real(r4),intent(in)                :: rv3
real(r4),intent(in)                :: rv4
character(len=8),intent(in)        :: vname
!write(*,4100) vname,iv1,iv2,rv3,rv4
write(iunit,4100) vname,iv1,iv2,rv3,rv4
4100 format(a,': layer,tlevel,range = ',i9,2x,i3,2x,1p2e16.7,2x,1p2e16.7)
end subroutine







end module mabio
