MODULE mgridinfo
USE mkinds
USE mconstants

IMPLICIT NONE

! public module variables
INTEGER(i4),PUBLIC                            :: nx
INTEGER(i4),PUBLIC                            :: ny
INTEGER(i4),PUBLIC                            :: nz
INTEGER(r4),PUBLIC                            :: nhybrd
INTEGER(r4),PUBLIC                            :: nsigma
REAL(r4),   PUBLIC                            :: dp00
REAL(r4),   PUBLIC                            :: dp00x
REAL(r4),   PUBLIC                            :: dp00f
REAL(r4),   PUBLIC                            :: ds00
REAL(r4),   PUBLIC                            :: ds00x
REAL(r4),   PUBLIC                            :: ds00f
REAL(r4),   PUBLIC                            :: dp00i
REAL(r4),   PUBLIC                            :: isotop
INTEGER(i4),PUBLIC                            :: kapref
INTEGER(i4),PUBLIC                            :: vsigma
REAL(r4),   PUBLIC                            :: thref
INTEGER(i4),   PUBLIC                         :: thflag
INTEGER(i4),   PUBLIC                         :: ilspec
REAL(r4),   PUBLIC                            :: thbase
INTEGER(i4),ALLOCATABLE,PUBLIC                :: layer(:)
REAL(r4),ALLOCATABLE,PUBLIC                   :: dp0k(:)
REAL(r4),ALLOCATABLE,PUBLIC                   :: ds0k(:)
REAL(r4),ALLOCATABLE,PUBLIC                   :: target_densities(:)
REAL(r4),ALLOCATABLE,PUBLIC                   :: isosigma(:,:,:)
REAL(r4),ALLOCATABLE,PUBLIC                   :: mdepth(:,:)
REAL(r4),ALLOCATABLE,PUBLIC                   :: mplon(:,:)
REAL(r4),ALLOCATABLE,PUBLIC                   :: mplat(:,:)
REAL(r4),ALLOCATABLE,PUBLIC                   :: mulon(:,:)
REAL(r4),ALLOCATABLE,PUBLIC                   :: mulat(:,:)
REAL(r4),ALLOCATABLE,PUBLIC                   :: mvlon(:,:)
REAL(r4),ALLOCATABLE,PUBLIC                   :: mvlat(:,:)
REAL(r4),ALLOCATABLE,PUBLIC                   :: mqlon(:,:)
REAL(r4),ALLOCATABLE,PUBLIC                   :: mqlat(:,:)
REAL(r4),ALLOCATABLE,PUBLIC                   :: mpasp(:,:)
REAL(r4),ALLOCATABLE,PUBLIC                   :: mpang(:,:)
REAL(r4),ALLOCATABLE,PUBLIC                   :: muscx(:,:)
REAL(r4),ALLOCATABLE,PUBLIC                   :: muscy(:,:)
REAL(r4),ALLOCATABLE,PUBLIC                   :: mvscx(:,:)
REAL(r4),ALLOCATABLE,PUBLIC                   :: mvscy(:,:)
REAL(r4),ALLOCATABLE,PUBLIC                   :: mpscx(:,:)
REAL(r4),ALLOCATABLE,PUBLIC                   :: mpscy(:,:)
REAL(r4),ALLOCATABLE,PUBLIC                   :: mqscx(:,:)
REAL(r4),ALLOCATABLE,PUBLIC                   :: mqscy(:,:)
REAL(r4),ALLOCATABLE,PUBLIC                   :: fcori(:,:)
REAL(r4),ALLOCATABLE,PUBLIC                   :: tbaric(:,:)
REAL(r4),ALLOCATABLE,PUBLIC                   :: depth(:,:)
! private module variables

LOGICAL(bl),private                           :: blkdatPresent
LOGICAL(bl),private                           :: regionalGridPresent
LOGICAL(bl),private                           :: regionalDepthPresent
LOGICAL(bl),private                           :: romsGrdPresent
CHARACTER(:),ALLOCATABLE,PUBLIC               :: fcmodel

integer(i4)                                   :: i

CONTAINS

        SUBROUTINE inquireGridType(fcmodel)
                IMPLICIT NONE

                CHARACTER(:),ALLOCATABLE, INTENT(OUT)          :: fcmodel

                INQUIRE(FILE="blkdat.input", EXIST=blkdatPresent)
                INQUIRE(FILE="regional.grid.a", EXIST=regionalGridPresent)
                INQUIRE(FILE="regional.depth.a", EXIST=regionalDepthPresent)
                INQUIRE(FILE="roms_grd.nc", EXIST=romsGrdPresent)

                IF(blkdatPresent .and. regionalGridPresent .and. regionalDepthPresent) THEN

                        fcmodel='hycom'

                ELSE IF (romsGrdPresent) THEN
                        fcmodel='roms'

                ELSE
                        Write(*,'(A)') "Input not available"

                ENDIF

        END SUBROUTINE


        SUBROUTINE readGridInfo(fcmodel)
                IMPLICIT NONE

                CHARACTER(LEN=*)                           :: fcmodel


                IF(fcmodel == 'hycom') THEN

                        CALL readHYCOMGridInfo()

                ELSE IF (fcmodel == 'roms') THEN

                        CALL readROMSGridInfo()

                ELSE

                        WRITE(*,'(a)') 'Unsupported Forecast Model'    

                ENDIF


        END SUBROUTINE


        SUBROUTINE readHYCOMGridInfo()
                USE mkinds
                USE mabio
                use mncio
                USE mhycom

                IMPLICIT NONE

                ! Local Variables

                INTEGER(i4)                            :: k
                INTEGER(i4)                            :: iunit
                INTEGER(i4)                            :: fid
                CHARACTER(:),ALLOCATABLE               :: filename
                REAL(r4)                               :: hmin
                REAL(r4)                               :: hmax


                Write(*,'(A)') "Writing HYCOM Grid Info to gridinfo.nc"

                CALL readFromBlkdat("idm",nx)
                CALL readFromBlkdat("jdm",ny)
                CALL readFromBlkdat("kdm",nz)
                CALL readFromBlkdat("nhybrd",nhybrd)
                CALL readFromBlkdat("nsigma",nsigma)
                CALL readFromBlkdat("thermobaric ref. state ",kapref)
                CALL readFromBlkdat("reference pressure flag",thflag)
                CALL readFromBlkdat("reference density (sigma units)",thbase)
                CALL readFromBlkdat("spacially varying isopycnal target densities",vsigma)

                ALLOCATE(target_densities(nz))
                ALLOCATE(dp0k(nz))

                IF(nsigma==0) then
                        nsigma=1
                ENDIF
                ALLOCATE(ds0k(nsigma))
                ALLOCATE(depth(nx,ny))
                ALLOCATE(mdepth(nx,ny))
                ALLOCATE(mplon(nx,ny))
                ALLOCATE(mplat(nx,ny))
                ALLOCATE(mulon(nx,ny))
                ALLOCATE(mulat(nx,ny))
                ALLOCATE(mvlon(nx,ny))
                ALLOCATE(mvlat(nx,ny))
                ALLOCATE(mqlon(nx,ny))
                ALLOCATE(mqlat(nx,ny))
                ALLOCATE(mpang(nx,ny))
                ALLOCATE(mpasp(nx,ny))
                ALLOCATE(mpscx(nx,ny))
                ALLOCATE(mpscy(nx,ny))
                ALLOCATE(muscx(nx,ny))
                ALLOCATE(muscy(nx,ny))
                ALLOCATE(mvscx(nx,ny))
                ALLOCATE(mvscy(nx,ny))
                ALLOCATE(mqscx(nx,ny))
                ALLOCATE(mqscy(nx,ny))
                ALLOCATE(fcori(nx,ny))
                ALLOCATE(tbaric(nx,ny))
                ALLOCATE(isosigma(nx,ny,nz))

                dp0k=0.0
                ds0k=0.0

                CALL readTargetDensities(nz,target_densities)
                CALL read_hybrid_min_thickness(nz,nsigma,dp0k,ds0k)
                ilspec=1
                IF (sum(dp0k) < 0.001 ) THEN
                        ilspec=0
                        CALL realFromBlkdat("deep    z-level spacing minimum thickness (m)",dp00)
                        CALL readFromBlkdat("deep    z-level spacing maximum thickness (m)",dp00x)
                        CALL readFromBlkdat("deep    z-level spacing stretching factor",dp00f)
                        CALL readFromBlkdat("shallow z-level spacing minimum thickness (m)",ds00)
                        CALL readFromBlkdat("shallow z-level spacing maximum thickness (m)",ds00x)
                        CALL readFromBlkdat("shallow z-level spacing stretching factor",ds00f)
                        CALL readFromBlkdat("deep iso-pycnal spacing minimum thickness (m)",dp00i)
                        CALL hybrid_min_thickness(nhybrd,nsigma,dp00,dp00x,dp00f,ds00,ds00x,ds00f,dp0k,ds0k)

                ENDIF

                ! read regional grid

                filename="regional.grid"
                iunit=11

                CALL zaiost(nx,ny)
                CALL zaiopn(FileName,iunit)
                CALL zaiord(iunit,mplon,hmin,hmax)
                CALL zaiord(iunit,mplat,hmin,hmax)
                CALL zaiord(iunit,mqlon,hmin,hmax)
                CALL zaiord(iunit,mqlon,hmin,hmax)
                CALL zaiord(iunit,mulon,hmin,hmax)
                CALL zaiord(iunit,mulon,hmin,hmax)
                CALL zaiord(iunit,mvlon,hmin,hmax)
                CALL zaiord(iunit,mvlon,hmin,hmax)
                CALL zaiord(iunit,mpang,hmin,hmax)
                CALL zaiord(iunit,mpscx,hmin,hmax)
                CALL zaiord(iunit,mpscy,hmin,hmax)
                CALL zaiord(iunit,mqscx,hmin,hmax)
                CALL zaiord(iunit,mqscy,hmin,hmax)
                CALL zaiord(iunit,muscx,hmin,hmax)
                CALL zaiord(iunit,muscy,hmin,hmax)
                CALL zaiord(iunit,mvscx,hmin,hmax)
                CALL zaiord(iunit,mvscy,hmin,hmax)
                CALL zaiord(iunit,fcori,hmin,hmax)
                CALL zaiord(iunit,mpasp,hmin,hmax)
                CALL zaiocl(iunit)

 
                ! convert to 0-360.0

                !where (mplon<0.0) mplon=mplon+360.0

                ! read regional depth

                filename="regional.depth"
                iunit=11
                CALL zaiost(nx,ny)
                CALL zaiopn(FileName,iunit)
                CALL zaiord(iunit,depth,hmin,hmax)
                CALL zaiocl(iunit)

                mdepth=depth
                ! set thermobaric parameter

                IF (kapref==-1) THEN
                        filename="tbaric"
                        iunit=11
                        CALL zaiost(nx,ny)
                        CALL zaiopn(FileName,iunit)
                        CALL zaiord(iunit,tbaric,hmin,hmax)
                        CALL zaiocl(iunit)
                ELSE

                        tbaric=real(kapref,r4)

                ENDIF


                ! set spatially varying sigmas

                IF (vsigma==1) THEN
                        filename="iso.sigma"
                        iunit=11
                        CALL zaiost(nx,ny)
                        CALL zaiopn(FileName,iunit)
                        do i=1,nz
                        CALL zaiord(iunit,isosigma(:,:,i),hmin,hmax)
                        enddo
                        CALL zaiocl(iunit)

                        !call nciopn(filename,iunit)
                        !call nciorv(filename,iunit,"isosigma",isosigma)
                        !call nciocl(filename,iunit)
                ELSE
                        do i=1,nz
                        isosigma(:,:,i)=target_densities(i)
                        enddo

                ENDIF

        END SUBROUTINE


        SUBROUTINE readROMSGridINFO
                !! STUB code

        END SUBROUTINE


        ! writeGridInfo -- Write grid information to netcdf file

        SUBROUTINE writeGridInfo(fcmodel)

                USE mkinds
                USE mncio

                IMPLICIT NONE

                ! ARGUMENTS

                CHARACTER(LEN=*), INTENT(IN)           :: fcmodel

                ! Local Variables
                INTEGER(i4)                            :: fid
                CHARACTER(:),ALLOCATABLE               :: filename
                REAL(r4)                               :: hmin
                REAL(r4)                               :: hmax
                CHARACTER(:),ALLOCATABLE               :: vname
                INTEGER(i4)                            :: vtype
                INTEGER(i4)                            :: vdims(2),vdim(1),vdims3(3)
                INTEGER(i4),ALLOCATABLE                :: xc(:)
                INTEGER(i4),ALLOCATABLE                :: yc(:)
                INTEGER(i4),ALLOCATABLE                :: zc(:)
                INTEGER(i4)                            :: k

                ALLOCATE(xc(nx))
                ALLOCATE(yc(ny))
                ALLOCATE(zc(nz))

                DO k=1,nx
                xc(k)=k
                ENDDO

                DO k=1,ny
                yc(k)=k
                ENDDO

                DO k=1,nz
                zc(k)=k
                ENDDO

                !create file

                FileName="gridinfo.nc"

                CALL nciocf(filename,fid)

                !create dimension and coordinate variable 1

                vname="xc"
                vtype=NF90_Int
                vdim(1)=1
                CALL nciodd(filename,fid,vname,nx)
                CALL nciocv(filename,fid,vname,vtype,vdim)
                CALL nciowa(filename,fid,vname,'long_name','x-coordinate index in model grid')
                CALL nciowa(filename,fid,vname,'standard_name','x-coordinate index in model grid')
                CALL nciowa(filename,fid,vname,'axis','X')
                CALL nciowa(filename,fid,vname,'units','')

                !create dimension and coordinate variable 2

                vname="yc"
                vtype=NF90_Int
                vdim(1)=2
                CALL nciodd(filename,fid,vname,ny)
                CALL nciocv(filename,fid,vname,vtype,vdim)
                CALL nciowa(filename,fid,vname,'long_name','y-coordinate index in model grid')
                CALL nciowa(filename,fid,vname,'standard_name','y-coordinate index in model grid')
                CALL nciowa(filename,fid,vname,'axis','Y')
                CALL nciowa(filename,fid,vname,'units','')

                !create dimension and coordinate variable 3

                vname="zc"
                vtype=NF90_Int
                vdim(1)=3
                CALL nciodd(filename,fid,vname,nz)
                CALL nciocv(filename,fid,vname,vtype,vdim)
                CALL nciowa(filename,fid,vname,'long_name','generalized vertical coordinate layer')
                CALL nciowa(filename,fid,vname,'standard_name','generalized vertical cooridnate layer')
                CALL nciowa(filename,fid,vname,'axis','Z')
                CALL nciowa(filename,fid,vname,'units','')
                CALL nciowa(filename,fid,vname,'positive','down')

                ! create file variables

                vname="target_densities"
                vtype=NF90_Float
                vdim(1)=3
                CALL nciocv(filename,fid,vname,vtype,vdim)
                CALL nciowa(filename,fid,vname,'long_name','target isopycnal densities')
                CALL nciowa(filename,fid,vname,'standard_name','target isopycnal densities')
                CALL nciowa(filename,fid,vname,'units','')

    vname="hybrid_minimum_thickness"
    vtype=NF90_Float
    vdim(1)=3
    CALL nciocv(filename,fid,vname,vtype,vdim)
    CALL nciowa(filename,fid,vname,'long_name','hybrid minimum thickness')
    CALL nciowa(filename,fid,vname,'standard_name','hybrid_layer_minimum_thickness')
    CALL nciowa(filename,fid,vname,'units','m')    

    vname="shallow_hybrid_minimum_thickness"
    vtype=NF90_Float
    vdim(1)=3
    CALL nciocv(filename,fid,vname,vtype,vdim)
    CALL nciowa(filename,fid,vname,'long_name','shallow_hybrid minimum thickness')
    CALL nciowa(filename,fid,vname,'standard_name','shallow_z_levels_hybrid_layer_minimum_thickness')
    CALL nciowa(filename,fid,vname,'units','m') 

    vname="mplon"
    vtype=NF90_Float
    vdims=(/1,2/)
    CALL nciocv(filename,fid,vname,vtype,vdims)
    CALL nciowa(filename,fid,vname,'long_name','longitude at p points')
    CALL nciowa(filename,fid,vname,'standard_name','longitude at p points')
    CALL nciowa(filename,fid,vname,'units','degrees_east')

    vname="mplat"
    vtype=NF90_Float
    vdims=(/1,2/)
    CALL nciocv(filename,fid,vname,vtype,vdims)
    CALL nciowa(filename,fid,vname,'long_name','latitude at p points')
    CALL nciowa(filename,fid,vname,'standard_name','latitude at p points')
    CALL nciowa(filename,fid,vname,'units','degrees_north')

    vname="mqlon"
    vtype=NF90_Float
    vdims=(/1,2/)
    CALL nciocv(filename,fid,vname,vtype,vdims)
    CALL nciowa(filename,fid,vname,'long_name','longitude at q points')
    CALL nciowa(filename,fid,vname,'standard_name','longitude at q points')
    CALL nciowa(filename,fid,vname,'units','degrees_east')

    vname="mqlat"
    vtype=NF90_Float
    vdims=(/1,2/)
    CALL nciocv(filename,fid,vname,vtype,vdims)
    CALL nciowa(filename,fid,vname,'long_name','latitude at q points')
    CALL nciowa(filename,fid,vname,'standard_name','latitude at q points')
    CALL nciowa(filename,fid,vname,'units','degrees_north')

    vname="mulon"
    vtype=NF90_Float
    vdims=(/1,2/)
    CALL nciocv(filename,fid,vname,vtype,vdims)
    CALL nciowa(filename,fid,vname,'long_name','longitude at u points')
    CALL nciowa(filename,fid,vname,'standard_name','longitude at u points')
    CALL nciowa(filename,fid,vname,'units','degrees_east')

    vname="mulat"
    vtype=NF90_Float
    vdims=(/1,2/)
    CALL nciocv(filename,fid,vname,vtype,vdims)
    CALL nciowa(filename,fid,vname,'long_name','latitude at u points')
    CALL nciowa(filename,fid,vname,'standard_name','latitude at u points')
    CALL nciowa(filename,fid,vname,'units','degrees_north')

    vname="mvlon"
    vtype=NF90_Float
    vdims=(/1,2/)
    CALL nciocv(filename,fid,vname,vtype,vdims)
    CALL nciowa(filename,fid,vname,'long_name','longitude at v points')
    CALL nciowa(filename,fid,vname,'standard_name','longitude at v points')
    CALL nciowa(filename,fid,vname,'units','degrees_east')

    vname="mvlat"
    vtype=NF90_Float
    vdims=(/1,2/)
    CALL nciocv(filename,fid,vname,vtype,vdims)
    CALL nciowa(filename,fid,vname,'long_name','latitude at v points')
    CALL nciowa(filename,fid,vname,'standard_name','latitude at v points')
    CALL nciowa(filename,fid,vname,'units','degrees_north')

    vname="mpang"
    vtype=NF90_Float
    vdims=(/1,2/)
    CALL nciocv(filename,fid,vname,vtype,vdims)
    CALL nciowa(filename,fid,vname,'long_name','angle between x axis and east')
    CALL nciowa(filename,fid,vname,'standard_name','angle between x axis and east')
    CALL nciowa(filename,fid,vname,'units','degrees')

    vname="mpscx"
    vtype=NF90_Float
    vdims=(/1,2/)
    CALL nciocv(filename,fid,vname,vtype,vdims)
    CALL nciowa(filename,fid,vname,'long_name','metric factor for p points in x direction')
    CALL nciowa(filename,fid,vname,'standard_name','metric factor for p points in x direction')
    CALL nciowa(filename,fid,vname,'units','meters')

    vname="mpscy"
    vtype=NF90_Float
    vdims=(/1,2/)
    CALL nciocv(filename,fid,vname,vtype,vdims)
    CALL nciowa(filename,fid,vname,'long_name','metric factor for p points in y direction')
    CALL nciowa(filename,fid,vname,'standard_name','metric factor for p points in y direction')
    CALL nciowa(filename,fid,vname,'units','meters')

    vname="mqscx"
    vtype=NF90_Float
    vdims=(/1,2/)
    CALL nciocv(filename,fid,vname,vtype,vdims)
    CALL nciowa(filename,fid,vname,'long_name','metric factor for q points in x direction')
    CALL nciowa(filename,fid,vname,'standard_name','metric factor for q points in x direction')
    CALL nciowa(filename,fid,vname,'units','meters')

    vname="mqscy"
    vtype=NF90_Float
    vdims=(/1,2/)
    CALL nciocv(filename,fid,vname,vtype,vdims)
    CALL nciowa(filename,fid,vname,'long_name','metric factor for q points in y direction')
    CALL nciowa(filename,fid,vname,'standard_name','metric factor for q points in y direction')
    CALL nciowa(filename,fid,vname,'units','meters')

    vname="muscx"
    vtype=NF90_Float
    vdims=(/1,2/)
    CALL nciocv(filename,fid,vname,vtype,vdims)
    CALL nciowa(filename,fid,vname,'long_name','metric factor for u points in x direction')
    CALL nciowa(filename,fid,vname,'standard_name','metric factor for u points in x direction')
    CALL nciowa(filename,fid,vname,'units','meters')

    vname="muscy"
    vtype=NF90_Float
    vdims=(/1,2/)
    CALL nciocv(filename,fid,vname,vtype,vdims)
    CALL nciowa(filename,fid,vname,'long_name','metric factor for u points in y direction')
    CALL nciowa(filename,fid,vname,'standard_name','metric factor for u points in y direction')
    CALL nciowa(filename,fid,vname,'units','meters')

    vname="mvscx"
    vtype=NF90_Float
    vdims=(/1,2/)
    CALL nciocv(filename,fid,vname,vtype,vdims)
    CALL nciowa(filename,fid,vname,'long_name','metric factor for v points in x direction')
    CALL nciowa(filename,fid,vname,'standard_name','metric factor for v points in x direction')
    CALL nciowa(filename,fid,vname,'units','meters')

    vname="mvscy"
    vtype=NF90_Float
    vdims=(/1,2/)
    CALL nciocv(filename,fid,vname,vtype,vdims)
    CALL nciowa(filename,fid,vname,'long_name','metric factor for v points in y direction')
    CALL nciowa(filename,fid,vname,'standard_name','metric factor for v points in y direction')
    CALL nciowa(filename,fid,vname,'units','meters')

    vname="fcori"
    vtype=NF90_Float
    vdims=(/1,2/)
    CALL nciocv(filename,fid,vname,vtype,vdims)
    CALL nciowa(filename,fid,vname,'long_name','Coriolis parameter')
    CALL nciowa(filename,fid,vname,'standard_name','Coriolis parameter')
    CALL nciowa(filename,fid,vname,'units','second-1')

    vname="mpasp"
    vtype=NF90_Float
    vdims=(/1,2/)
    CALL nciocv(filename,fid,vname,vtype,vdims)
    CALL nciowa(filename,fid,vname,'long_name','model grid mesh aspect ratio')
    CALL nciowa(filename,fid,vname,'standard_name','model grid mesh aspect ratio')
    CALL nciowa(filename,fid,vname,'units','')

    vname="mdepth"
    vtype=NF90_Float
    vdims=(/1,2/)
    CALL nciocv(filename,fid,vname,vtype,vdims)
    CALL nciowa(filename,fid,vname,'long_name','model bathymetry')
    CALL nciowa(filename,fid,vname,'standard_name','model bathymetry')
    CALL nciowa(filename,fid,vname,'units','meters')
    call nciowa(filename,fid,vname,'_FillValue',2.00**100)

    vname="tbaric"
    vtype=NF90_Float
    vdims=(/1,2/)
    CALL nciocv(filename,fid,vname,vtype,vdims)
    CALL nciowa(filename,fid,vname,'long_name','Thermobaric Parameter')
    CALL nciowa(filename,fid,vname,'standard_name','Thermobaric Parameter')
    CALL nciowa(filename,fid,vname,'units','-')
    call nciowa(filename,fid,vname,'_FillValue',2.00**100)

    vname="isosigma"
    vtype=NF90_Float
    vdims3=(/1,2,3/)
    CALL nciocv(filename,fid,vname,vtype,vdims3)
    CALL nciowa(filename,fid,vname,'long_name','Spatially Varying Target Densities')
    CALL nciowa(filename,fid,vname,'standard_name','target densities')
    CALL nciowa(filename,fid,vname,'units','-')
    call nciowa(filename,fid,vname,'_FillValue',2.00**100)
    
! put global atributes -- we dump all grid related parametere here

    CALL nciowa(filename,fid,"nhybrd",nhybrd)
    CALL nciowa(filename,fid,"nsigma",nsigma)
    CALL nciowa(filename,fid,"dp00",dp00)
    CALL nciowa(filename,fid,"dp00x",dp00x)
    CALL nciowa(filename,fid,"dp00f",dp00f)
    CALL nciowa(filename,fid,"ds00",ds00)
    CALL nciowa(filename,fid,"ds00x",ds00x)
    CALL nciowa(filename,fid,"ds00f",ds00f)
    CALL nciowa(filename,fid,"dp00i",dp00i)
    CALL nciowa(filename,fid,"kapref",kapref)
    CALL nciowa(filename,fid,"thbase",thbase)
    CALL nciowa(filename,fid,"thflag",thflag)
    CALL nciowa(filename,fid,"ilspec",ilspec)

! end netcdf define mode

    CALL ncioed(filename,fid)

! write data into variables

    CALL nciowv(filename,fid,"xc",xc)
    CALL nciowv(filename,fid,"yc",yc)
    CALL nciowv(filename,fid,"zc",zc)
    CALL nciowv(filename,fid,"target_densities",target_densities)
    CALL nciowv(filename,fid,"hybrid_minimum_thickness",dp0k)
    CALL nciowv(filename,fid,"shallow_hybrid_minimum_thickness",ds0k)
    CALL nciowv(filename,fid,"mplon",mplon)
    CALL nciowv(FileName,fid,"mplat",mplat)
    CALL nciowv(filename,fid,"mqlon",mqlon)
    CALL nciowv(FileName,fid,"mqlat",mqlat)
    CALL nciowv(filename,fid,"mulon",mulon)
    CALL nciowv(FileName,fid,"mulat",mulat)
    CALL nciowv(filename,fid,"mvlon",mvlon)
    CALL nciowv(FileName,fid,"mvlat",mvlat)
    CALL nciowv(filename,fid,"mpang",mpang)
    CALL nciowv(FileName,fid,"mpscx",mpscx)
    CALL nciowv(FileName,fid,"mpscy",mpscy)
    CALL nciowv(FileName,fid,"mqscx",mqscx)
    CALL nciowv(FileName,fid,"mqscy",mqscy)
    CALL nciowv(FileName,fid,"muscx",muscx)
    CALL nciowv(FileName,fid,"muscy",muscy)
    CALL nciowv(FileName,fid,"mvscx",mvscx)
    CALL nciowv(FileName,fid,"mvscy",mvscy)
    CALL nciowv(FileName,fid,"fcori",fcori)
    CALL nciowv(FileName,fid,"mpasp",mpasp)
    CALL nciowv(FileName,fid,"mdepth",depth)
    CALL nciowv(FileName,fid,"tbaric",tbaric)
    CALL nciowv(FileName,fid,"isosigma",isosigma)

    ! close file

    CALL nciocl(filename,fid)

END SUBROUTINE

END MODULE mgridinfo
