module miotsis
use mkinds
use mncio
use mgrid
use mpobs

contains

subroutine write_tsis_pobs_sst(cdtg,nsst,lon,lat,sst,err)
IMPLICIT NONE
character(len=*)               :: cdtg
character(:),allocatable       :: filename
character(:),allocatable       :: vname
integer(i4)                    :: fid,i,j,iunit,ntyp,nsst
integer(i4)                    :: vtype
integer(i4)                    :: vdim(1),vdims2(2)
real(r4)                       :: lon(nsst),lat(nsst),sst(nsst),err(nsst)
integer(i4)                    :: xc(nsst)
do i=1,nsst
xc(i)=i
enddo

filename="sst_obs_"//cdtg//".nc"

 !create file

call nciocf(filename,fid)
vname="xc"
vtype=NF90_int
vdim(1)=1
call nciodd(filename,fid,vname,nsst)
call nciocv(filename,fid,vname,vtype,vdim)
call nciowa(filename,fid,vname,'long_name','obs_number')
call nciowa(filename,fid,vname,'standard_name','obs_number')

!create data variable 1
vname="lon"
vtype=NF90_FLOAT
vdim=(/1/)
call nciocv(filename,fid,vname,vtype,vdim)
call nciowa(filename,fid,vname,'long_name','longitude')
call nciowa(filename,fid,vname,'standard_name','longitude')
call nciowa(filename,fid,vname,'units','degrees_east')

!create data variable 2
vname="lat"
vtype=NF90_FLOAT
vdim=(/1/)
call nciocv(filename,fid,vname,vtype,vdim)
call nciowa(filename,fid,vname,'long_name','latitude')
call nciowa(filename,fid,vname,'standard_name','latitude')
call nciowa(filename,fid,vname,'units','degrees_north')

!create data variable 3
vname="sst"
vtype=NF90_FLOAT
vdim=(/1/)
call nciocv(filename,fid,vname,vtype,vdim)
call nciowa(filename,fid,vname,'long_name','observation value')
call nciowa(filename,fid,vname,'standard_name','observation value')

!create data variable 4

vname="err"
vtype=NF90_FLOAT
vdim=(/1/)
call nciocv(filename,fid,vname,vtype,vdim)
call nciowa(filename,fid,vname,'long_name','observation err')
call nciowa(filename,fid,vname,'standard_name','observation error')
!end define mode

call ncioed(filename,fid)

! write data into variables
call nciowv(filename,fid,"xc",xc)
call nciowv(filename,fid,"lon",lon)
call nciowv(filename,fid,"lat",lat)
call nciowv(filename,fid,"sst",sst)
call nciowv(filename,fid,"err",err)

! close file

call nciocl(filename,fid)


end subroutine write_tsis_pobs_sst

subroutine write_tsis_profiles(cdtg,nprofiles,nlevels,ntyp,glon,glat,nx,ny,pfx,clim)

IMPLICIT NONE
character(len=*)               :: cdtg
character(len=3)               :: typ
character(len=5)               :: pfx
character(:),allocatable       :: filename
character(:),allocatable       :: vname
integer(i4)                    :: fid,i,j,iunit,ntyp,nprofiles,nlevels
integer(i4)                    :: vtype
integer(i4)                    :: vdims(3),vdim(1),vdims2(2)
integer(i4),allocatable        :: ityp(:),iprofnum(:),ilevel(:),ob_grp_present(:),iob_grp(:)
real(r4)                       :: glon(nx,ny),glat(nx,ny),xc(nx),yc(ny)
integer(i4)                    :: nx,ny,nob_grp
integer(i4),optional           :: clim

do i=1,nx
xc(i)=i
enddo

do j=1,ny
yc(j)=j
enddo


nob_grp=4

allocate(ityp(ntyp))
allocate(iprofnum(nprofiles))
allocate(ilevel(nlevels))
allocate(iob_grp(nob_grp))
allocate(ob_grp_present(nob_grp))

ob_grp_present=0
if(nprofiles>1) ob_grp_present(1)=1

do i=1,ntyp
   ityp(i)=i
enddo

do i=1,nlevels
   ilevel(i)=i
enddo

do i=1,nprofiles
   iprofnum(i)=i
enddo

do i=1,nob_grp
   iob_grp(i)=i
enddo

if(clim==1) then
 filename=trim(tsis_obs_location)//"tsis_cobs_"//trim(pfx)//"_"//cdtg//".nc"
else
 filename=trim(tsis_obs_location)//"tsis_obs_"//trim(pfx)//"_"//cdtg//".nc"  
endif        


 !create file

call nciocf(filename,fid)
!create dimension and coordinate variable 1
vname="obs_typ"
vtype=NF90_int
vdim(1)=1
call nciodd(filename,fid,vname,ntyp)
call nciocv(filename,fid,vname,vtype,vdim)
call nciowa(filename,fid,vname,'long_name','observation_type')
call nciowa(filename,fid,vname,'standard_name','observation_type')

vname="obs_level"
vtype=NF90_int
vdim(1)=2
call nciodd(filename,fid,vname,nlevels)
call nciocv(filename,fid,vname,vtype,vdim)
call nciowa(filename,fid,vname,'long_name','observation_level')
call nciowa(filename,fid,vname,'standard_name','observation_level')

vname="prof_num"
vtype=NF90_int
vdim(1)=3
call nciodd(filename,fid,vname,nprofiles)
call nciocv(filename,fid,vname,vtype,vdim)
call nciowa(filename,fid,vname,'long_name','profile_number')
call nciowa(filename,fid,vname,'standard_name','profile_number')

! gridded lon/lat
!vname="glon"
vname="xc"
vtype=NF90_FLOAT
vdim(1)=4
call nciodd(filename,fid,vname,nx)
call nciocv(filename,fid,vname,vtype,vdim)
call nciowa(filename,fid,vname,'long_name','longitude')
call nciowa(filename,fid,vname,'standard_name','longitude')
call nciowa(filename,fid,vname,'units','degrees east')

!vname="glat"
vname="yc"
vtype=NF90_FLOAT
vdim(1)=5
call nciodd(filename,fid,vname,ny)
call nciocv(filename,fid,vname,vtype,vdim)
call nciowa(filename,fid,vname,'long_name','latitude')
call nciowa(filename,fid,vname,'standard_name','latitude')
call nciowa(filename,fid,vname,'units','degrees north')

vname="ob_grp"
vtype=NF90_INT
vdim(1)=6
call nciodd(filename,fid,vname,nob_grp)
call nciocv(filename,fid,vname,vtype,vdim)
call nciowa(filename,fid,vname,'long_name','observation group')

!create data variable 1
vname="lon"
vtype=NF90_FLOAT
vdims=(/1,2,3/)
call nciocv(filename,fid,vname,vtype,vdims)
call nciowa(filename,fid,vname,'long_name','longitude')
call nciowa(filename,fid,vname,'standard_name','longitude')
call nciowa(filename,fid,vname,'units','degrees_east')

!create data variable 4
vname="lat"
vtype=NF90_FLOAT
vdims=(/1,2,3/)
call nciocv(filename,fid,vname,vtype,vdims)
call nciowa(filename,fid,vname,'long_name','latitude')
call nciowa(filename,fid,vname,'standard_name','latitude')
call nciowa(filename,fid,vname,'units','degrees_north')

!create data variable 5
!vname="time"
!vtype=NF90_FLOAT
!vdims=(/1,2,3/)
!call nciocv(filename,fid,vname,vtype,vdims)
!call nciowa(filename,fid,vname,'long_name','time')
!call nciowa(filename,fid,vname,'standard_name','time')
!call nciowa(filename,fid,vname,'units','days since 1900-12-31 00:00:00')

!create data variable 6
vname="val"
vtype=NF90_FLOAT
vdims=(/1,2,3/)
call nciocv(filename,fid,vname,vtype,vdims)
call nciowa(filename,fid,vname,'long_name','observation value')
call nciowa(filename,fid,vname,'standard_name','observation value')

!create data variable 7
vname="err"
vtype=NF90_FLOAT
vdims=(/1,2,3/)
call nciocv(filename,fid,vname,vtype,vdims)
call nciowa(filename,fid,vname,'long_name','observation err')
call nciowa(filename,fid,vname,'standard_name','observation error')

!create data variable 8
!vname="bkg"
!vtype=NF90_FLOAT
!vdims=(/1,2,3/)
!call nciocv(filename,fid,vname,vtype,vdims)
!call nciowa(filename,fid,vname,'long_name','forecast value')
!call nciowa(filename,fid,vname,'standard_name','forecast value')
!call nciowa(filename,fid,vname,'units','standard')

!create data variable 9
!vname="inov"
!vtype=NF90_FLOAT
!vdims=(/1,2,3/)
!call nciocv(filename,fid,vname,vtype,vdims)
!call nciowa(filename,fid,vname,'long_name','innovation')
!call nciowa(filename,fid,vname,'standard_name','innovation')
!call nciowa(filename,fid,vname,'units','standard')

!create data variable 10
!vname="residual"
!vtype=NF90_FLOAT
!vdims=(/1,2,3/)
!call nciocv(filename,fid,vname,vtype,vdims)
!call nciowa(filename,fid,vname,'long_name','analysis_residual')
!call nciowa(filename,fid,vname,'standard_name','analysis_residual')
!call nciowa(filename,fid,vname,'units','standard')
!end define mode

!create data variable 10
vname="grdi"
vtype=NF90_FLOAT
vdims=(/1,2,3/)
call nciocv(filename,fid,vname,vtype,vdims)
call nciowa(filename,fid,vname,'long_name','model grid index')
call nciowa(filename,fid,vname,'standard_name','x-grid-index')

!create data variable 10
vname="grdj"
vtype=NF90_FLOAT
vdims=(/1,2,3/)
call nciocv(filename,fid,vname,vtype,vdims)
call nciowa(filename,fid,vname,'long_name','y-model grid index')
call nciowa(filename,fid,vname,'standard_name','y-grid-index')

!create data variable 10
vname="id"
vtype=NF90_INT
vdims=(/1,2,3/)
call nciocv(filename,fid,vname,vtype,vdims)
call nciowa(filename,fid,vname,'long_name','obs_flag')
call nciowa(filename,fid,vname,'standard_name','obs_flag')

!create data variable 10
vname="ob_grp_present"
vtype=NF90_INT
vdim=(/6/)
call nciocv(filename,fid,vname,vtype,vdim)
call nciowa(filename,fid,vname,'standard_name','obs_groups_present')

!data variable 10
vname="ssh"
vtype=NF90_float
vdims2=(/4,5/)
call nciocv(filename,fid,vname,vtype,vdims2)
call nciowa(filename,fid,vname,'long_name','sea_surface_height')
call nciowa(filename,fid,vname,'standard_name','sea_surface_height')
call nciowa(filename,fid,vname,'units','m')
call nciowa(filename,fid,vname,'_FillValue',2.00**100)

!data variable 10
vname="ssh_err"
vtype=NF90_float
vdims2=(/4,5/)
call nciocv(filename,fid,vname,vtype,vdims2)
call nciowa(filename,fid,vname,'long_name','sea_surface_height_err')
call nciowa(filename,fid,vname,'standard_name','sea_surface_height_err')
call nciowa(filename,fid,vname,'units','m')
call nciowa(filename,fid,vname,'_FillValue',2.00**100)

!data variable 10
vname="sst"
vtype=NF90_float
vdims2=(/4,5/)
call nciocv(filename,fid,vname,vtype,vdims2)
call nciowa(filename,fid,vname,'long_name','sea_surface_temperature')
call nciowa(filename,fid,vname,'standard_name','sea_surface_temperature')
call nciowa(filename,fid,vname,'units','degc')
call nciowa(filename,fid,vname,'_FillValue',2.00**100)

vname="sst_err"
vtype=NF90_float
vdims2=(/4,5/)
call nciocv(filename,fid,vname,vtype,vdims2)
call nciowa(filename,fid,vname,'long_name','sea_surface_temperature_err')
call nciowa(filename,fid,vname,'standard_name','sea_surface_temperature_err')
call nciowa(filename,fid,vname,'units','degc')
call nciowa(filename,fid,vname,'_FillValue',2.00**100)
!data variable 10
vname="sss"
vtype=NF90_float
vdims2=(/4,5/)
call nciocv(filename,fid,vname,vtype,vdims2)
call nciowa(filename,fid,vname,'long_name','sea_surface_salinity')
call nciowa(filename,fid,vname,'standard_name','sea_surface_salinity')
call nciowa(filename,fid,vname,'units','-')
call nciowa(filename,fid,vname,'_FillValue',2.00**100)
vname="sss_err"
vtype=NF90_float
vdims2=(/4,5/)
call nciocv(filename,fid,vname,vtype,vdims2)
call nciowa(filename,fid,vname,'long_name','sea_surface_salinity_err')
call nciowa(filename,fid,vname,'standard_name','sea_surface_salinity_err')
call nciowa(filename,fid,vname,'units','-')
call nciowa(filename,fid,vname,'_FillValue',2.00**100)

!data variable 11
vname="uc"
vtype=NF90_float
vdims2=(/4,5/)
call nciocv(filename,fid,vname,vtype,vdims2)
call nciowa(filename,fid,vname,'long_name','eastward_velocity')
call nciowa(filename,fid,vname,'standard_name','eastward_velocity')
call nciowa(filename,fid,vname,'units','m/s')
call nciowa(filename,fid,vname,'_FillValue',2.00**100)
vname="vc"
vtype=NF90_float
vdims2=(/4,5/)
call nciocv(filename,fid,vname,vtype,vdims2)
call nciowa(filename,fid,vname,'long_name','northward_velocity')
call nciowa(filename,fid,vname,'standard_name','northward_velocity')
call nciowa(filename,fid,vname,'units','m/s')
call nciowa(filename,fid,vname,'_FillValue',2.00**100)

!data variable 10
vname="av_ssh"
vtype=NF90_float
vdims2=(/4,5/)
call nciocv(filename,fid,vname,vtype,vdims2)
call nciowa(filename,fid,vname,'long_name','aviso_sea_surface_height_mapped')
call nciowa(filename,fid,vname,'standard_name','aviso_sea_surface_height_mapped')
call nciowa(filename,fid,vname,'units','m')
call nciowa(filename,fid,vname,'_FillValue',2.00**100)
vname="av_ssh_err"
vtype=NF90_float
vdims2=(/4,5/)
call nciocv(filename,fid,vname,vtype,vdims2)
call nciowa(filename,fid,vname,'long_name','aviso_sea_surface_height_err')
call nciowa(filename,fid,vname,'standard_name','avisosea_surface_height_err')
call nciowa(filename,fid,vname,'units','m')
call nciowa(filename,fid,vname,'_FillValue',2.00**100)


!end define mode

call ncioed(filename,fid)

! write data into variables
call nciowv(filename,fid,"prof_num",iprofnum)
call nciowv(filename,fid,"obs_typ",ityp)
call nciowv(filename,fid,"obs_level",ilevel)
call nciowv(filename,fid,"ob_grp",iob_grp)
!call nciowv(filename,fid,"glon",glon)
!call nciowv(filename,fid,"glat",glat)

call nciowv(filename,fid,"xc",xc)
call nciowv(filename,fid,"yc",yc)

call nciowv(filename,fid,"lon",profile_obs%lon)
call nciowv(filename,fid,"lat",profile_obs%lat)
!call nciowv(filename,fid,"time",profile_obs%time)
call nciowv(filename,fid,"val",profile_obs%val)
call nciowv(filename,fid,"err",profile_obs%err)
call nciowv(filename,fid,"grdi",profile_obs%gridi)
call nciowv(filename,fid,"grdj",profile_obs%gridj)
call nciowv(filename,fid,"id",profile_obs%id)
call nciowv(filename,fid,"ob_grp_present",ob_grp_present)
!call nciowv(filename,fid,"bkg",obs%bkg)
!call nciowv(filename,fid,"inov",obs%inov)
!call nciowv(filename,fid,"residual",obs%res)

! close file

call nciocl(filename,fid)

end subroutine write_tsis_profiles

subroutine write_argo_tsidb(filename,ncount,ntzlevs,prof_num,tzlevels,lon,lat,ptime,&
                            temp,saln,ctemp,csaln,ctstd,csstd)
implicit none
character(*)       :: filename
integer(i4)        :: ncount,ntzlevs,prof_num(ncount)
real(r4)           :: tzlevels(ntzlevs)
real(r4)           :: lon(ncount),lat(ncount),ptime(ncount)
real(r4)           :: temp(ncount,ntzlevs),saln(ncount,ntzlevs)
real(r4)           :: ctemp(ncount,ntzlevs),csaln(ncount,ntzlevs)
real(r4)           :: ctstd(ncount,ntzlevs),csstd(ncount,ntzlevs)

integer(i4)               :: fid
character(:), allocatable :: vname
integer(i4)               :: vtype
integer(i4)               :: vdims3(3),vdims2(2),vdim(1)


call nciocf(filename,fid)

vname="nprof"
vtype=NF90_INT
vdim(1)=1
call nciodd(filename,fid,vname,ncount)
call nciocv(filename,fid,vname,vtype,vdim)
call nciowa(filename,fid,vname,'long_name','profile_number')
call nciowa(filename,fid,vname,'standard_name','profile_number')

vname="z"
vtype=NF90_Float
vdim(1)=2
call nciodd(filename,fid,vname,ntzlevs)
call nciocv(filename,fid,vname,vtype,vdim)
call nciowa(filename,fid,vname,'long_name','pressure')
call nciowa(filename,fid,vname,'standard_name','pressure')
call nciowa(filename,fid,vname,'units','db')

!create data variable 1
vname="lon"
vtype=NF90_Float
vdim=(/1/)
call nciocv(filename,fid,vname,vtype,vdim)
call nciowa(filename,fid,vname,'long_name','longitude')

vname="lat"
vtype=NF90_Float
vdim=(/1/)
call nciocv(filename,fid,vname,vtype,vdim)
call nciowa(filename,fid,vname,'long_name','latitude')

vname="time"
vtype=NF90_Float
vdim=(/1/)
call nciocv(filename,fid,vname,vtype,vdim)
call nciowa(filename,fid,vname,'long_name','time')
call nciowa(filename,fid,vname,'standard_name','time')
call nciowa(filename,fid,vname,'units','days since 1900-12-31')
call nciowa(filename,fid,vname,'_FillValue',-999.0)

!create data variable 2
vname="temp"
vtype=NF90_Float
vdims2=(/1,2/)
call nciocv(filename,fid,vname,vtype,vdims2)
call nciowa(filename,fid,vname,'long_name','Potential Temperature')
call nciowa(filename,fid,vname,'standard_name','Potential Temperature')
call nciowa(filename,fid,vname,'units','degC')
call nciowa(filename,fid,vname,'_FillValue',-999.0)

!create data variable 3
vname="saln"
vtype=NF90_Float
vdims2=(/1,2/)
call nciocv(filename,fid,vname,vtype,vdims2)
call nciowa(filename,fid,vname,'long_name','Salinity')
call nciowa(filename,fid,vname,'standard_name','Salinity')
call nciowa(filename,fid,vname,'units','psu')
call nciowa(filename,fid,vname,'_FillValue',-999.0)

!create data variable 4
vname="ctemp"
vtype=NF90_Float
vdims2=(/1,2/)
call nciocv(filename,fid,vname,vtype,vdims2)
call nciowa(filename,fid,vname,'long_name','Climatological Potential Temperature')
call nciowa(filename,fid,vname,'standard_name','Climatological Potential Temperature')
call nciowa(filename,fid,vname,'units','degC')
call nciowa(filename,fid,vname,'_FillValue',-999.0)

!create data variable 3
vname="csaln"
vtype=NF90_Float
vdims2=(/1,2/)
call nciocv(filename,fid,vname,vtype,vdims2)
call nciowa(filename,fid,vname,'long_name','Climatological Salinity')
call nciowa(filename,fid,vname,'standard_name','Climatological Salinity')
call nciowa(filename,fid,vname,'units','psu')
call nciowa(filename,fid,vname,'_FillValue',-999.0)

vname="ctstd"
vtype=NF90_Float
vdims2=(/1,2/)
call nciocv(filename,fid,vname,vtype,vdims2)
call nciowa(filename,fid,vname,'long_name','climatological temp std dev')
call nciowa(filename,fid,vname,'standard_name','climatological temp std dev.')
call nciowa(filename,fid,vname,'units','degC')
call nciowa(filename,fid,vname,'_FillValue',-999.0)

vname="csstd"
vtype=NF90_Float
vdims2=(/1,2/)
call nciocv(filename,fid,vname,vtype,vdims2)
call nciowa(filename,fid,vname,'long_name','climatological saln std dev')
call nciowa(filename,fid,vname,'standard_name','climatological saln std dev.')
call nciowa(filename,fid,vname,'units','psu')
call nciowa(filename,fid,vname,'_FillValue',-999.0)

! end define mode

call ncioed(filename,fid)


! write data into variables
call nciowv(filename,fid,"nprof",prof_num)
call nciowv(filename,fid,"z",tzlevels)
call nciowv(filename,fid,"lon",lon)
call nciowv(filename,fid,"lat",lat)
call nciowv(filename,fid,"time",ptime)
call nciowv(filename,fid,"temp",temp)
call nciowv(filename,fid,"saln",saln)
call nciowv(filename,fid,"ctemp",ctemp)
call nciowv(filename,fid,"csaln",csaln)
call nciowv(filename,fid,"ctstd",ctstd)
call nciowv(filename,fid,"csstd",csstd)

! close file
call nciocl(filename,fid)
end subroutine







subroutine write_tsis_summary(cdtg)

IMPLICIT NONE
character(len=*)               :: cdtg
character(len=3)               :: typ
character(:),allocatable       :: filename
character(:),allocatable       :: vname
integer(i4)                    :: fid,i,j,iunit
integer(i4)                    :: vtype
integer(i4)                    :: vdims(1),vdim(1)
integer(i4),allocatable        :: ityp(:),obsnum(:)



allocate(ityp(ntobs))
allocate(obsnum(ntobs))

! replace obs_typ with integer types
do i=1,ntobs
   obsnum(i)=i
   if(trim(gobs(i)%typ) .eq. "sla") then
         ityp(i)=1
   else if (trim(gobs(i)%typ) .eq. "sst") then
         ityp(i)=2
   else if (trim(gobs(i)%typ) .eq. "tem") then
         ityp(i)=3
   else if (trim(gobs(i)%typ) .eq. "sal") then
         ityp(i)=4
   else if (trim(gobs(i)%typ) .eq. "den") then
         ityp(i)=5
   else if (trim(gobs(i)%typ) .eq. "thk") then
         ityp(i)=6
   else if (trim(gobs(i)%typ) .eq. "uvl") then
         ityp(i)=7
   else if (trim(gobs(i)%typ) .eq. "vvl") then
         ityp(i)=8
   endif

enddo

filename="tsisout_"//cdtg//".nc"
!create file

call nciocf(filename,fid)
!create dimension and coordinate variable 1
vname="obsnum"
vtype=NF90_int
vdim(1)=1
call nciodd(filename,fid,vname,ntobs)
call nciocv(filename,fid,vname,vtype,vdim)
call nciowa(filename,fid,vname,'long_name','observation_number')
call nciowa(filename,fid,vname,'standard_name','observation_number')
call nciowa(filename,fid,vname,'units','none')

!create data variable 1
vname="obs_type"
vtype=NF90_INT
vdims=(/1/)
call nciocv(filename,fid,vname,vtype,vdims)
call nciowa(filename,fid,vname,'long_name','obs_type')
call nciowa(filename,fid,vname,'standard_name','obs_type')

!create data variable 2
vname="level"
vtype=NF90_INT
vdims=(/1/)
call nciocv(filename,fid,vname,vtype,vdims)
call nciowa(filename,fid,vname,'long_name','level')
call nciowa(filename,fid,vname,'standard_name','level')

!create data variable 3
vname="lon"
vtype=NF90_FLOAT
vdims=(/1/)
call nciocv(filename,fid,vname,vtype,vdims)
call nciowa(filename,fid,vname,'long_name','longitude')
call nciowa(filename,fid,vname,'standard_name','longitude')
call nciowa(filename,fid,vname,'units','degrees_east')

!create data variable 4
vname="lat"
vtype=NF90_FLOAT
vdims=(/1/)
call nciocv(filename,fid,vname,vtype,vdims)
call nciowa(filename,fid,vname,'long_name','latitude')
call nciowa(filename,fid,vname,'standard_name','latitude')
call nciowa(filename,fid,vname,'units','degrees_north')

!create data variable 5
vname="time"
vtype=NF90_FLOAT
vdims=(/1/)
call nciocv(filename,fid,vname,vtype,vdims)
call nciowa(filename,fid,vname,'long_name','time')
call nciowa(filename,fid,vname,'standard_name','time')
call nciowa(filename,fid,vname,'units','days since 1900-12-31 00:00:00')

!create data variable 6
vname="val"
vtype=NF90_FLOAT
vdims=(/1/)
call nciocv(filename,fid,vname,vtype,vdims)
call nciowa(filename,fid,vname,'long_name','observation value')
call nciowa(filename,fid,vname,'standard_name','observation value')
call nciowa(filename,fid,vname,'units','standard')

!create data variable 7
vname="err"
vtype=NF90_FLOAT
vdims=(/1/)
call nciocv(filename,fid,vname,vtype,vdims)
call nciowa(filename,fid,vname,'long_name','observation err')
call nciowa(filename,fid,vname,'standard_name','observation error')
call nciowa(filename,fid,vname,'units','standard')

!create data variable 8
vname="bkg"
vtype=NF90_FLOAT
vdims=(/1/)
call nciocv(filename,fid,vname,vtype,vdims)
call nciowa(filename,fid,vname,'long_name','forecast value')
call nciowa(filename,fid,vname,'standard_name','forecast value')
call nciowa(filename,fid,vname,'units','standard')

!create data variable 9
vname="inov"
vtype=NF90_FLOAT
vdims=(/1/)
call nciocv(filename,fid,vname,vtype,vdims)
call nciowa(filename,fid,vname,'long_name','innovation')
call nciowa(filename,fid,vname,'standard_name','innovation')
call nciowa(filename,fid,vname,'units','standard')

!create data variable 10
vname="residual"
vtype=NF90_FLOAT
vdims=(/1/)
call nciocv(filename,fid,vname,vtype,vdims)
call nciowa(filename,fid,vname,'long_name','analysis_residual')
call nciowa(filename,fid,vname,'standard_name','analysis_residual')
call nciowa(filename,fid,vname,'units','standard')
!end define mode
call ncioed(filename,fid)

! write data into variables
call nciowv(filename,fid,"obsnum",obsnum)
call nciowv(filename,fid,"obs_type",ityp)
call nciowv(filename,fid,"lon",gobs(1:ntobs)%lon)
call nciowv(filename,fid,"lat",gobs(1:ntobs)%lat)
call nciowv(filename,fid,"level",gobs(1:ntobs)%lvl)
call nciowv(filename,fid,"time",gobs(1:ntobs)%time)
call nciowv(filename,fid,"val",gobs(1:ntobs)%val)
call nciowv(filename,fid,"err",gobs(1:ntobs)%err)
call nciowv(filename,fid,"bkg",gobs(1:ntobs)%bkg)
call nciowv(filename,fid,"inov",gobs(1:ntobs)%inov)
call nciowv(filename,fid,"residual",gobs(1:ntobs)%res)

! close file

call nciocl(filename,fid)

end subroutine write_tsis_summary

subroutine ncfldio(filename,lon,lat,level,time,var,varname,std_name,units,nx,ny,nz,nt)

character(*)  :: filename,varname,std_name,units
character(:),allocatable :: vname
integer(i4)   :: nx,ny,nz,nt
real(r4)      :: lon(nx),lat(ny),var(nx,ny,nz,1)
real(r4)      :: time(nt), level(nz)

integer(i4)               :: fid,i,j,iunit
integer(i4)               :: vtype
integer(i4)               :: vdims(4),vdim(1)

!create file
      call nciocf(filename,fid)

      !create dimension and coordinate variable 1
      vname="lon"
      vtype=NF90_Float
      vdim(1)=1
      call nciodd(filename,fid,vname,nx)
      call nciocv(filename,fid,vname,vtype,vdim)
      call nciowa(filename,fid,vname,'long_name','Longitude')
      call nciowa(filename,fid,vname,'standard_name','Longitude')
      call nciowa(filename,fid,vname,'units','degrees_east')
      
      !create dimension and coordinate variable 2
      vname="lat"
      vtype=NF90_Float
      vdim(1)=2
      call nciodd(filename,fid,vname,ny)
      call nciocv(filename,fid,vname,vtype,vdim)
      call nciowa(filename,fid,vname,'long_name','Latitude')
      call nciowa(filename,fid,vname,'standard_name','Latitude')
      call nciowa(filename,fid,vname,'units','degrees_north')

      !create dimension and coordinate variable 2
      vname="level"
      vtype=NF90_Float
      vdim(1)=3
      call nciodd(filename,fid,vname,nz)
      call nciocv(filename,fid,vname,vtype,vdim)
      call nciowa(filename,fid,vname,'long_name','level')
      call nciowa(filename,fid,vname,'standard_name','level')
      call nciowa(filename,fid,vname,'units','m')

      !create dimension and coordinate variable 2
      vname="time"
      vtype=NF90_Float
      vdim(1)=4
      call nciodd(filename,fid,vname,nt)
      call nciocv(filename,fid,vname,vtype,vdim)
      call nciowa(filename,fid,vname,'long_name','time')
      call nciowa(filename,fid,vname,'standard_name','time')
      call nciowa(filename,fid,vname,'units','days since 1900-12-31 00:00:00')
      
      !create data variable 1
      vtype=NF90_Float
      vdims=(/1,2,3,4/)
      call nciocv(filename,fid,varname,vtype,vdims)
      call nciowa(filename,fid,varname,'long_name',std_name)
      call nciowa(filename,fid,varname,'standard_name',std_name)
      call nciowa(filename,fid,varname,'units',units)
      call nciowa(filename,fid,varname,'_FillValue',2.00**100)

      call ncioed(filename,fid)

      ! write data into variables
      call nciowv(filename,fid,"lon",lon)
      call nciowv(filename,fid,"lat",lat)
      call nciowv(filename,fid,"level",level)
      call nciowv(filename,fid,"time",time)
      call nciowv(filename,fid,varname,var)

      ! close file

      call nciocl(filename,fid)

end subroutine

subroutine ncuvp(filename,lon,lat,level,time,var,varname,std_name,nx,ny,nz,nt)

character(*)  :: filename,varname,std_name
character(:),allocatable :: vname
integer(i4)   :: nx,ny,nz,nt
real(r4)      :: lon(nx),lat(ny),var(nx,ny,nz,1)
real(r4)      :: time(nt), level(nz)

integer(i4)               :: fid,i,j,iunit
integer(i4)               :: vtype
integer(i4)               :: vdims(4),vdim(1)

!create file
      call nciocf(filename,fid)

      !create dimension and coordinate variable 1
      vname="lon"
      vtype=NF90_Float
      vdim(1)=1
      call nciodd(filename,fid,vname,nx)
      call nciocv(filename,fid,vname,vtype,vdim)
      call nciowa(filename,fid,vname,'long_name','Longitude')
      call nciowa(filename,fid,vname,'standard_name','Longitude')
      call nciowa(filename,fid,vname,'units','degrees_east')
      
      !create dimension and coordinate variable 2
      vname="lat"
      vtype=NF90_Float
      vdim(1)=2
      call nciodd(filename,fid,vname,ny)
      call nciocv(filename,fid,vname,vtype,vdim)
      call nciowa(filename,fid,vname,'long_name','Latitude')
      call nciowa(filename,fid,vname,'standard_name','Latitude')
      call nciowa(filename,fid,vname,'units','degrees_north')

      !create dimension and coordinate variable 2
      vname="level"
      vtype=NF90_Float
      vdim(1)=3
      call nciodd(filename,fid,vname,nz)
      call nciocv(filename,fid,vname,vtype,vdim)
      call nciowa(filename,fid,vname,'long_name','level')
      call nciowa(filename,fid,vname,'standard_name','level')
      call nciowa(filename,fid,vname,'units','m')

      !create dimension and coordinate variable 2
      vname="time"
      vtype=NF90_Float
      vdim(1)=4
      call nciodd(filename,fid,vname,nt)
      call nciocv(filename,fid,vname,vtype,vdim)
      call nciowa(filename,fid,vname,'long_name','time')
      call nciowa(filename,fid,vname,'standard_name','time')
      call nciowa(filename,fid,vname,'units','days since 1900-12-31 00:00:00')
      
      !create data variable 1
      vtype=NF90_Float
      vdims=(/1,2,3,4/)
      call nciocv(filename,fid,varname,vtype,vdims)
      call nciowa(filename,fid,varname,'long_name',std_name)
      call nciowa(filename,fid,varname,'standard_name',std_name)
      call nciowa(filename,fid,varname,'_FillValue',2.00**100)

      call ncioed(filename,fid)

      ! write data into variables
      call nciowv(filename,fid,"lon",lon)
      call nciowv(filename,fid,"lat",lat)
      call nciowv(filename,fid,"level",level)
      call nciowv(filename,fid,"time",time)
      call nciowv(filename,fid,varname,var)

      ! close file

      call nciocl(filename,fid)

end subroutine









subroutine ncfldij(filename,var,varname,std_name,units,nx,ny)

character(*)  :: filename,varname,std_name,units
character(:),allocatable :: vname
integer(i4)   :: nx,ny
real(r4)      :: var(nx,ny)
real(r4),allocatable :: xc(:),yc(:)

integer(i4)               :: fid,i,j,iunit
integer(i4)               :: vtype
integer(i4)               :: vdims(2),vdim(1)


allocate(xc(nx),yc(ny))

do i=1,nx
   xc(i)=i
enddo

do j=1,ny
   yc(j)=j
enddo


!create file
      call nciocf(filename,fid)

      !create dimension and coordinate variable 1
      vname="xc"
      vtype=NF90_Float
      vdim(1)=1
      call nciodd(filename,fid,vname,nx)
      call nciocv(filename,fid,vname,vtype,vdim)
      call nciowa(filename,fid,vname,'long_name','Longitude')
      call nciowa(filename,fid,vname,'standard_name','Longitude')
      call nciowa(filename,fid,vname,'units','degrees_east')
!create dimension and coordinate variable 2
      vname="yc"
      vtype=NF90_Float
      vdim(1)=2
      call nciodd(filename,fid,vname,ny)
      call nciocv(filename,fid,vname,vtype,vdim)
      call nciowa(filename,fid,vname,'long_name','Latitude')
      call nciowa(filename,fid,vname,'standard_name','Latitude')
      call nciowa(filename,fid,vname,'units','degrees_north')

      !create data variable 1
      vtype=NF90_Float
      vdims=(/1,2/)
      call nciocv(filename,fid,varname,vtype,vdims)
      call nciowa(filename,fid,varname,'long_name',std_name)
      call nciowa(filename,fid,varname,'standard_name',std_name)
      call nciowa(filename,fid,varname,'units',units)
      call nciowa(filename,fid,varname,'_FillValue',2.00**100)

      call ncioed(filename,fid)

      ! write data into variables
      call nciowv(filename,fid,"xc",xc)
      call nciowv(filename,fid,"yc",yc)
      call nciowv(filename,fid,varname,var)

      ! close file

      call nciocl(filename,fid)

end subroutine








end module
